@@ -11,7 +11,7 @@
 #              with the application.
 # ----------------------------------------------------------------------------
 # $Source: src/decimalfp/__init__.py $
-# $Revision: 2021-04-17T17:47:23+02:00 $
+# $Revision: 2021-11-19T11:09:46+01:00 $
 
 
 # noinspection PyUnresolvedReferences
@@ -222,7 +222,7 @@ also support all rounding modes mentioned above.
     Decimal('12')
 """
 
-__version__ = 0, 11, 4
+__version__ = 0, 12, 2
 
 # Under PyPy the C implementation is slower than the Python
 # implementation, so we force to import the latter.
@@ -238,6 +238,11 @@ import os  # noqa: I100, I202
 _force_python_impl = os.getenv('DECIMALFP_FORCE_PYTHON_IMPL')
 del os
 
+from fractions import Fraction
+# patch Fraction, so that issubclass(Fraction, SupportsAsIntergerRatio) is True
+if not hasattr(Fraction, 'as_integer_ratio'):
+    Fraction.as_integer_ratio = lambda f: (f.numerator, f.denominator)
+
 if _impl == 'PyPy' or _force_python_impl:
     from ._pydecimalfp import (
         Decimal, get_dflt_rounding_mode, ROUNDING, set_dflt_rounding_mode,
