@@ -9,7 +9,7 @@ License:     This program is part of a larger application. For license
              with the application.
 ------------------------------------------------------------------------------
 $Source: src/decimalfp/_cdecimalfp.c $
-$Revision: 2021-04-14T22:07:35+02:00 $
+$Revision: 2021-07-02T16:40:27+02:00 $
 */
 
 #define PY_SSIZE_T_CLEAN
@@ -1082,8 +1082,7 @@ fpdec_from_number(fpdec_t *tmp, PyObject *obj) {
         if (rc == FPDEC_OK)
             fpdec = tmp;
     }
-    else if (PyObject_IsInstance(obj, Real) ||
-             PyObject_IsInstance(obj, StdLibDecimal)) {
+    else if (PyObject_HasAttrString(Py_TYPE(obj), "as_integer_ratio")) {
         ASSIGN_AND_CHECK_NULL(ratio,
                               PyObject_CallMethod(obj, "as_integer_ratio",
                                                   NULL));
